// 代码复杂度检测
const { ESLint } = require("eslint");

const FUNCTION_TYPE = [
  "Arrow function",
  "Function",
  "Method",
  "Async function",
];

const eslint = new ESLint({
  overrideConfig: {
    rules: {
      complexity: ["error", { max: 0 }],
    },
  },
  useEslintrc: true,
});

// 获取文件位置
const getFilePath = (filePath) => {
  return filePath.replace(process.cwd(), "").trim();
};

// 获取函数类型
const getFunctionType = (msg) => {
  return FUNCTION_TYPE.find((type) => msg.includes(type)) || "";
};

// 获取函数名称
const getFunctionName = (msg) => {
  const name = /'([a-zA-Z0-9_$]+)'/g.exec(msg);
  return name ? name[0] : "*";
};

// 获取代码复杂度
const getComplexity = (msg) => {
  const complexity = /(\d+)\./g.exec(msg);
  return complexity ? complexity[0].replace(".", "") : "";
};

// 格式化 complexity message 信息
const BASE_COMPLEXITY_MESSAGE = "has a complexity of";
const SUFFIX_COMPLEXITY_MESSAGE = "Maximum allowed is 0.";
const formatComplexityMessage = (message) => {
  const fmtMsg = message
    .replace(SUFFIX_COMPLEXITY_MESSAGE, "")
    .replace(BASE_COMPLEXITY_MESSAGE, "");
  return {
    funcType: getFunctionType(fmtMsg),
    funcName: getFunctionName(fmtMsg),
    complexity: getComplexity(fmtMsg),
  };
};

/**
 *
 * @param {*} filePaths
 * @param {*} min
 * 复杂度检测
 */
const executeOnFiles = async (files) => {
  try {
    const filesInfo = await eslint.lintFiles(files);
    let funcCount = 0;
    let otherCount = 0;
    const detail = filesInfo
      .map((fileInfo) => {
        const { messages, filePath } = fileInfo;
        return messages
          .map((msg) => {
            const { ruleId, message, line, endLine } = msg;
            if (ruleId === "complexity") {
              const { funcType, funcName, complexity } =
                formatComplexityMessage(message);
              funcType ? funcCount++ : otherCount++;
              return {
                complexity: +complexity,
                funcName,
                funcType,
                funcLoc: `${getFilePath(filePath)}[${line}:${endLine}]`,
                filePath: getFilePath(filePath),
              };
            }
            return "";
          })
          .filter(Boolean);
      })
      .flat();
    return {
      fileCount: filesInfo.length,
      funcCount,
      otherCount,
      detail,
    };
  } catch (error) {
    console.error("complexity", error);
    process.exit(1);
  }
};

/**
 *
 * @param {*} files scan 扫描出来的文件
 */
module.exports = function complexity(files) {
  return executeOnFiles(files);
};
